#include <gui/model/Model.hpp>
#include <gui/model/ModelListener.hpp>

#ifndef SIMULATOR

	#include "stm32f769i_discovery.h"
	#include "FreeRTOS.h"
	#include "queue.h"
	#include "task.h"
	#include "ledtask.h"

	//*******************************************************
	//   Define Queue handles
	//*******************************************************
	extern "C" xQueueHandle led_msg_q; //From LEDTask
	xQueueHandle gui_msg_q;

#else
	#include <stdio.h>
#endif

Model::Model()
    : modelListener(0)
    , led_state(false)
{
	
	#ifndef SIMULATOR
	
	//*******************************************************
	// 
	//  Create GUI message queue
	//
	//  Check for messages from backend, with zero timeout
	//  Any time there's a message on this mailbox, react to it. value is meaningless in this case..
  //	
	//*******************************************************
	//
  gui_msg_q = xQueueGenericCreate(1, 1, 0);
	#endif
}

void Model::tick()
{

#ifndef SIMULATOR

	//*******************************************************
	// 
	//  HANDLE MESSAGES
	//
	//  Check for messages from backend, with zero timeout to 
	//  avoid blocking the UI. 
	// 
	//*******************************************************
	//
	
	uint8_t msg = 0;
	if (xQueueReceive(gui_msg_q, &msg, 0) == pdTRUE)
  {
  	// Queue is used as a flag, so don't check msg.
    // Full = PRESSED, Empty = Do nothing.		
     btnPressed();
  }

#endif
}

void Model::btnPressed()
{
    modelListener->btnPressed();
}

void Model::setLEDState(bool state)
{
	#ifdef SIMULATOR
	printf("LED State: %d\n", state);
	#endif
	
  led_state = state;
  updateLEDState();
}

void Model::updateLEDState()
{
#ifndef SIMULATOR
	uint8_t msg = LED_OFF;
	if (led_state)
  {
		msg = LED_ON;
	}
  else
	{
		msg = LED_OFF;
	}    

	if (led_msg_q)
  {
		xQueueSend(led_msg_q, &msg, 0);
  }
#endif
}
